from datetime import datetime
from os.path import abspath, dirname, join
import shutil
import tempfile
from unittest import TestCase

from freezegun import freeze_time
import pytest

from iatikit.data.registry import Registry
from iatikit.utils.exceptions import NoDataError
from iatikit.utils.config import CONFIG
import iatikit.data


class TestNoData(TestCase):
    def setUp(self):
        self.empty_path = tempfile.mkdtemp(dir=dirname(abspath(__file__)))
        config_dict = {'paths': {'registry': self.empty_path}}
        CONFIG.read_dict(config_dict)

    def test_no_data(self):
        with pytest.raises(NoDataError):
            Registry()

    def tearDown(self):
        shutil.rmtree(self.empty_path, ignore_errors=True)


class TestRegistry(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestRegistry, self).__init__(*args, **kwargs)
        registry_path = join(dirname(abspath(__file__)),
                             'fixtures', 'registry')
        config_dict = {'paths': {'registry': registry_path}}
        CONFIG.read_dict(config_dict)

    @freeze_time("2015-12-09 08:00:00")
    def test_last_updated(self):
        with pytest.warns(UserWarning, match=r'last updated 8 days ago'):
            registry = Registry()
        assert registry.last_updated == datetime(2015, 12, 1, 3, 57, 19)

    @freeze_time("2015-12-02")
    def test_publishers(self):
        publisher_names = ['fixture-org', 'old-org']
        registry = Registry()
        publishers = registry.publishers
        assert len(publishers) == 2
        assert publishers[0].name in publisher_names

    @freeze_time("2015-12-02")
    def test_datasets(self):
        dataset_names = [
            'fixture-org-activities',
            'fixture-org-activities2',
            'fixture-org-org',
            'old-org-acts',
            'old-org-missing-acts',
        ]
        registry = Registry()
        datasets = registry.datasets
        assert len(datasets) == 5
        for dataset in datasets:
            assert dataset.name in dataset_names

    @freeze_time("2015-12-02")
    def test_activities(self):
        activity_dataset_names = [
            'fixture-org-activities',
            'fixture-org-activities2',
            'old-org-acts',
            'old-org-missing-acts',
        ]
        registry = Registry()
        activities = registry.activities
        assert len(activities) == 6
        for activity in activities:
            assert activity.dataset.name in activity_dataset_names

    @freeze_time("2015-12-02")
    def test_shortcut(self):
        registry = iatikit.data()
        assert(len(registry.datasets)) == 5



class XPathQueryBuilder(object):
    def __init__(self, schema, prefix='', count=False):
        self._schema = schema
        self._count = count
        self._prefix = prefix

    def where(self, **kwargs):
        query_str = self._prefix

        exprs = []
        for shortcut, values in kwargs.items():
            if '__' in shortcut:
                shortcut, operator = shortcut.split('__')
            else:
                operator = 'eq'
            for value in values:
                expr = self.filter(shortcut, operator, value)
                exprs.append(expr)
        query_str += ''.join(['[{}]'.format(x) for x in exprs])
        if self._count:
            query_str = 'count({})'.format(query_str)
        return query_str

    def filter(self, shortcut, operator, value):
        return getattr(self._schema, shortcut)().where(operator, value)



from os.path import abspath, dirname, join
from unittest import TestCase

from freezegun import freeze_time
import pytest

from iatikit.data.registry import Registry
from iatikit.utils.exceptions import FilterError
from iatikit.utils.config import CONFIG


class TestGenericSet(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestGenericSet, self).__init__(*args, **kwargs)
        registry_path = join(dirname(abspath(__file__)),
                             'fixtures', 'registry')
        config_dict = {'paths': {'registry': registry_path}}
        CONFIG.read_dict(config_dict)

    @freeze_time("2015-12-02")
    def setUp(self):
        self.registry = Registry()

    def test_set_get(self):
        org_dataset = self.registry.datasets.get('fixture-org-org')
        assert org_dataset.name == 'fixture-org-org'

    def test_set_get_unknown(self):
        unknown_dataset = self.registry.datasets.get('unknown')
        assert unknown_dataset is None

    def test_set_find(self):
        org_dataset = self.registry.datasets.find(name='fixture-org-org')
        assert org_dataset.name == 'fixture-org-org'

    def test_set_not_found(self):
        with pytest.raises(IndexError):
            self.registry.datasets.find(name='not-a-dataset')

    def test_set_first(self):
        first_dataset = self.registry.datasets.first()
        zero_index_dataset = self.registry.datasets[0]
        assert first_dataset.name == zero_index_dataset.name

    def test_set_count(self):
        datasets = self.registry.datasets
        assert datasets.count() == 5

    def test_set_all(self):
        dataset_names = [
            'fixture-org-activities',
            'fixture-org-activities2',
            'fixture-org-org',
            'old-org-acts',
            'old-org-missing-acts',
        ]
        datasets = self.registry.datasets.all()
        assert isinstance(datasets, list)
        assert len(datasets) == 5
        for dataset in datasets:
            assert dataset.name in dataset_names

    def test_set_filter_chaining(self):
        act_datasets = self.registry.datasets.filter(filetype='activity')
        no_datasets = act_datasets.filter(name='fixture-org-org')
        assert no_datasets.count() == 0

        org_datasets = self.registry.datasets.filter(filetype='organisation')
        org_dataset = org_datasets.filter(name='fixture-org-org')
        assert org_dataset.count() == 1

    def test_set_where_chaining(self):
        act_datasets = self.registry.datasets.where(filetype='activity')
        no_datasets = act_datasets.where(name='fixture-org-org')
        assert no_datasets.count() == 0

        org_datasets = self.registry.datasets.where(filetype='organisation')
        org_dataset = org_datasets.where(name='fixture-org-org')
        assert org_dataset.count() == 1

    def test_set_unknown_filter(self):
        with pytest.raises(FilterError):
            self.registry.datasets.where(unknown_filter='unknown')



class SchemaError(Exception):
    """Raised when an unknown schema is requested."""
    pass


class FilterError(Exception):
    """Raised when an unknown or invalid set filter is used."""
    pass


class NoDataError(Exception):
    """Raised when no data is found in the local cache."""
    pass


class NoCodelistsError(Exception):
    """Raised when no codelists are found in the local cache."""
    pass


class UnknownSectorVocabError(Exception):
    """Raised when a sector vocabulary is used that is not on the
       IATI codelists.
    """
    pass


class UnknownSectorCodeError(Exception):
    """Raised when a sector code is used that is not on the
       IATI codelists.
    """
    pass


class InvalidSectorCodeError(Exception):
    """Raised when a sector is constructed using a non-sector codelist item
    """
    pass


class SchemaNotFoundError(Exception):
    pass


class MappingsNotFoundError(Exception):
    pass



from ..utils.exceptions import SchemaError
from ..utils.types import StringType


# pylint: disable=too-many-ancestors
class OrganisationSchema101(object):
    version = '1.01'

    @classmethod
    def id(cls):  # pylint: disable=invalid-name
        return cls.org_identifier()

    @classmethod
    def org_identifier(cls):
        return StringType('iati-identifier/text()')


class OrganisationSchema102(OrganisationSchema101):
    version = '1.02'


class OrganisationSchema103(OrganisationSchema102):
    version = '1.03'


class OrganisationSchema104(OrganisationSchema103):
    version = '1.04'


class OrganisationSchema105(OrganisationSchema104):
    version = '1.05'


class OrganisationSchema201(OrganisationSchema105):
    version = '2.01'

    @classmethod
    def org_identifier(cls):
        return StringType('organisation-identifier/text()')


class OrganisationSchema202(OrganisationSchema201):
    version = '2.02'


class OrganisationSchema203(OrganisationSchema202):
    version = '2.03'


def get_organisation_schema(version):
    schema = None

    if version == '2.03':
        schema = OrganisationSchema203
    elif version == '2.02':
        schema = OrganisationSchema202
    elif version == '2.01':
        schema = OrganisationSchema201
    elif version == '1.05':
        schema = OrganisationSchema105
    elif version == '1.04':
        schema = OrganisationSchema104
    elif version == '1.03':
        schema = OrganisationSchema103
    elif version == '1.02':
        schema = OrganisationSchema102
    elif version == '1.01':
        schema = OrganisationSchema101

    if not schema:
        msg = 'Unknown organisation schema: version: {}'.format(version)
        raise SchemaError(msg)

    return schema



from os.path import abspath, dirname, join
from unittest import TestCase

from mock import patch

from iatikit.data.dataset import DatasetSet, Dataset
from iatikit.utils.config import CONFIG


class TestDatasets(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestDatasets, self).__init__(*args, **kwargs)
        self.registry_path = join(dirname(abspath(__file__)),
                                  'fixtures', 'registry')

    def setUp(self):
        self.org_datasets = DatasetSet(
            join(self.registry_path, 'data', 'fixture-org', '*'),
            join(self.registry_path, 'metadata', 'fixture-org', '*'),
        )

    def test_datasets_iter(self):
        dataset_list = list(self.org_datasets)
        assert len(dataset_list) == 3

    def test_datasets_filter_by_filetype(self):
        act_datasets = self.org_datasets.where(filetype='activity').all()
        assert len(act_datasets) == 2
        assert act_datasets[0].name == 'fixture-org-activities'

    def test_datasets_filter_by_name(self):
        org_datasets = self.org_datasets.where(name='fixture-org-org').all()
        assert len(org_datasets) == 1
        assert org_datasets[0].name == 'fixture-org-org'


class TestDataset(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestDataset, self).__init__(*args, **kwargs)
        registry_path = join(dirname(abspath(__file__)),
                             'fixtures', 'registry')
        self.old_org_acts = Dataset(
            join(registry_path, 'data',
                 'old-org', 'old-org-acts.xml'),
            join(registry_path, 'metadata',
                 'old-org', 'old-org-acts.json'),
        )

        self.fixture_org_acts = Dataset(
            join(registry_path, 'data',
                 'fixture-org', 'fixture-org-activities.xml'),
            join(registry_path, 'metadata',
                 'fixture-org', 'fixture-org-activities.json'),
        )

        standard_path = join(dirname(abspath(__file__)),
                             'fixtures', 'standard')
        config_dict = {'paths': {'standard': standard_path}}
        CONFIG.read_dict(config_dict)

    def test_dataset_name(self):
        assert self.old_org_acts.name == 'old-org-acts'

    def test_dataset_version(self):
        assert self.old_org_acts.version == '1.03'

    def test_dataset_repr(self):
        dataset_repr = '<Dataset (old-org-acts)>'
        assert str(self.old_org_acts) == dataset_repr

    def test_dataset_validate_xml(self):
        assert bool(self.old_org_acts.validate_xml()) is True

    def test_dataset_validate_iati(self):
        assert bool(self.old_org_acts.validate_iati()) is True

    @patch('logging.Logger.warning')
    def test_dataset_validate_codelists_old(self, fake_logger_warning):
        assert bool(self.old_org_acts.validate_codelists()) is True
        msg = ('Can\'t perform codelist validation for ' +
               'IATI version %s datasets.', '1.03')
        fake_logger_warning.assert_called_once_with(*msg)

    def test_dataset_validate_codelists(self):
        result = self.fixture_org_acts.validate_codelists()
        assert result.is_valid is False
        assert len(result.errors) == 2
        err_msgs = [
            'The value "999" is not in the Sector Vocabulary codelist.',
            'The value "6" is not in the Activity Status codelist.',
        ]
        for error in result.errors:
            assert str(error) in err_msgs

    def test_dataset_root(self):
        assert self.old_org_acts.root == 'iati-activities'

    @patch('webbrowser.open_new_tab')
    def test_dataset_show(self, fake_open_new_tab):
        url = 'https://iatiregistry.org/dataset/old-org-acts'
        self.old_org_acts.show()
        fake_open_new_tab.assert_called_once_with(url)

    def test_activities(self):
        assert self.old_org_acts.activities.count() == 2
        activity = self.old_org_acts.activities[1]
        assert activity.id == 'NL-CHC-98765-NL-CHC-98765-XGG00NS00'

    def test_metadata(self):
        dataset_metadata = self.old_org_acts.metadata
        assert dataset_metadata.get('extras') \
            .get('publisher_organization_type') == '21'



import logging
from datetime import datetime

from ..data.sector import Sector
from ..standard.codelist import CodelistSet, CodelistItem
from ..utils.abstract import GenericType


class StringType(GenericType):
    def where(self, operation, value):
        if operation in ['contains', 'startswith']:
            if operation == 'startswith':
                operation = 'starts-with'
            return '{expr}[{operation}(., "{value}")]'.format(
                expr=self.get(),
                operation=operation,
                value=value,
            )
        return super(StringType, self).where(operation, value)


class DateType(GenericType):
    def where(self, operation, value):
        operator = {
            'lt': '<', 'lte': '<=',
            'gt': '>', 'gte': '>=',
            'eq': '=',
        }.get(operation)
        if operator:
            tmpl = 'number(translate({expr}, "-", "")) {operator} {value}'
            return tmpl.format(
                expr=self.get(),
                operator=operator,
                value=str(value).replace('-', ''),
            )
        return super(DateType, self).where(operation, value)

    def run(self, etree):
        dates = []
        dates_str = etree.xpath(self.get())
        for date_str in dates_str:
            try:
                dates.append(datetime.strptime(date_str, '%Y-%m-%d').date())
            except ValueError:
                logging.getLogger(__name__).warning(
                    'Invalid date: "%s"', date_str)
        return dates


class SectorType(GenericType):
    def __init__(self, expr, condition):
        super(SectorType, self).__init__(expr)
        self.condition = condition

    @staticmethod
    def _vocab_condition(conditions):
        conditions_list = []
        if not isinstance(conditions, list):
            conditions = [conditions]
        for condition in conditions:
            if condition is None:
                conditions_list.append('not(@vocabulary)')
            else:
                conditions_list.append('@vocabulary = "{}"'.format(condition))
        conditions_str = ' or '.join(conditions_list)
        if len(conditions_list) > 1:
            conditions_str = '({})'.format(conditions_str)
        return conditions_str

    def where(self, operation, value):
        if operation == 'in':
            if not isinstance(value, Sector) or value.vocabulary.code != '2':
                raise Exception('{} is not a sector category'.format(value))
            codelist_items = CodelistSet().get('Sector').where(
                category=value.code.code).all()
            conditions = ' or '.join(['@code = "{code}"'.format(code=c.code)
                                      for c in codelist_items])
            conditions = ['(' + conditions + ')']
            conditions.append(
                self._vocab_condition(self.condition.get('1')))
            return '{expr}[{conditions}]'.format(
                expr=self.get(),
                conditions=' and '.join(conditions),
            )
        elif operation == 'eq':
            conditions = []
            if not isinstance(value, Sector):
                raise Exception('{} is not a sector'.format(value))
            if isinstance(value.code, CodelistItem):
                code = value.code.code
            else:
                code = value.code
            if code is not None:
                conditions.append('@code = "{code}"'.format(code=code))
            if value.vocabulary is not None:
                conds = self.condition.get(value.vocabulary.code,
                                           value.vocabulary.code)
                conditions.append(self._vocab_condition(conds))
            return '{expr}[{conditions}]'.format(
                expr=self.get(),
                conditions=' and '.join(conditions),
            )
        return super(SectorType, self).where(operation, value)

    def run(self, etree):
        return [Sector(x.get('code'),
                       vocabulary=x.get('vocabulary', '1'),
                       percentage=x.get('percentage'))
                for x in etree.xpath(self.get())]


class XPathType(GenericType):
    def where(self, operation, value):
        return value


class BooleanType(GenericType):
    def run(self, etree):
        return etree.xpath('{expr} = "true" or {expr} = "1"'.format(
            expr=self.get(),
        ))

    def where(self, operation, value):
        if value is not bool(value):
            raise Exception('{} is not a boolean'.format(value))
        if value:
            return '{expr} = "true" or {expr} = "1"'.format(
                expr=self.get(),
            )
        else:
            return 'not({expr}) or {expr} = "false" or {expr} = "0"'.format(
                expr=self.get(),
            )



from collections import OrderedDict, defaultdict
import json
from os.path import exists, join
from os import makedirs, unlink as _unlink
import shutil
import logging
import zipfile

import requests
import unicodecsv as csv

from ..standard.codelist import CodelistSet
from .config import CONFIG
from . import helpers


def data():
    path = CONFIG['paths']['registry']
    # downloads from https://andylolz.github.io/iati-data-dump/
    data_url = 'https://www.dropbox.com/s/kkm80yjihyalwes/' + \
               'iati_dump.zip?dl=1'
    shutil.rmtree(path, ignore_errors=True)
    makedirs(path)
    zip_filepath = join(path, 'iati_dump.zip')

    logging.getLogger(__name__).info('Downloading all IATI registry data...')
    request = requests.get(data_url, stream=True)
    with open(zip_filepath, 'wb') as handler:
        shutil.copyfileobj(request.raw, handler)
    logging.getLogger(__name__).info('Unzipping data...')
    with zipfile.ZipFile(zip_filepath, 'r') as zip_ref:
        zip_ref.extractall(path)
    logging.getLogger(__name__).info('Cleaning up...')
    _unlink(zip_filepath)


def metadata():
    logging.getLogger(__name__).info(
        'Downloading metadata from the IATI registry...')
    path = join(CONFIG['paths']['registry'], 'metadata')
    shutil.rmtree(path, ignore_errors=True)
    makedirs(path)

    url_tmpl = 'https://iatiregistry.org/api/3/action/package_search' + \
               '?start={start}&rows=1000'
    org_url_tmpl = 'https://iatiregistry.org/api/3/action/group_show' + \
                   '?id={org_slug}'
    start = 0
    while True:
        j = requests.get(url_tmpl.format(start=start)).json()
        if len(j['result']['results']) == 0:
            break
        for res in j['result']['results']:
            org = res['organization']
            if not org:
                continue
            org_name = org['name']
            if not exists(join(path, org_name + '.json')):
                j = requests.get(org_url_tmpl.format(org_slug=org_name)).json()
                with open(join(path, org_name + '.json'), 'w') as f:
                    json.dump(j['result'], f)
            dataset_name = res['name']
            orgpath = join(path, org_name)
            makedirs(orgpath, exist_ok=True)
            with open(join(orgpath, dataset_name + '.json'), 'w') as f:
                json.dump(res, f)
        start += 1000


_VERY_OLD_IATI_VERSIONS = ['1.01', '1.02']
_VERY_OLD_CODELISTS_URL = 'http://codelists102.archive.iatistandard.org' + \
                         '/data/codelist.csv'
_VERY_OLD_CODELIST_TMPL = 'http://codelists102.archive.iatistandard.org' + \
                         '/data/codelist/{codelist_name}.csv'

_OLD_IATI_VERSIONS = ['1.03']
_OLD_CODELISTS_URL = 'http://codelists103.archive.iatistandard.org' + \
                    '/data/codelist.json'
_OLD_CODELIST_TMPL = 'http://codelists103.archive.iatistandard.org' + \
                    '/data/codelist/{codelist_name}.csv'

_NEW_CODELISTS_TMPL = 'https://iatistandard.org/' + \
                      'reference_downloads/{version}/' + \
                      'codelists/downloads/clv2/codelists.json'
_NEW_CODELIST_TMPL = 'https://iatistandard.org/' + \
                     'reference_downloads/{version}/' + \
                     'codelists/downloads/clv2/json/en/{codelist_name}.json'


def _get_codelist_mappings(versions):
    all_codelists = CodelistSet()

    def filter_complete(mapping):
        try:
            return all_codelists.get(mapping['codelist']).complete
        except:
            raise Exception(mapping['codelist'])

    path = join(CONFIG['paths']['standard'], 'codelist_mappings')
    shutil.rmtree(path, ignore_errors=True)
    makedirs(path)

    logging.getLogger(__name__).info(
        'Downloading IATI Standard codelist mappings...')

    tmpl = 'https://iatistandard.org/reference_downloads/' + \
           '{version}/codelists/downloads/clv2/mapping.json'
    for version in versions:
        if version not in ['1.01', '1.02', '1.03']:
            version_path = version.replace('.', '')
            mapping_path = join(path, version_path)
            makedirs(mapping_path)

            mapping_url = tmpl.format(version=version_path)
            mappings = requests.get(mapping_url).json()

            mappings = list(filter(filter_complete, mappings))

            activity_mappings = [
                x for x in mappings
                if not x['path'].startswith('//iati-org')]
            filepath = join(mapping_path, 'activity-mappings.json')
            with open(filepath, 'w') as handler:
                json.dump(activity_mappings, handler)

            organisation_mappings = [
                x for x in mappings
                if not x['path'].startswith('//iati-act')]
            filepath = join(mapping_path, 'organisation-mappings.json')
            with open(filepath, 'w') as handler:
                json.dump(organisation_mappings, handler)


def codelists():
    def get_list_of_codelists(version):
        if version in _VERY_OLD_IATI_VERSIONS:
            request = requests.get(_VERY_OLD_CODELISTS_URL)
            list_of_codelists = [x['name'] for x in csv.DictReader(
                [x for x in request.iter_lines()])]
        elif version in _OLD_IATI_VERSIONS:
            j = requests.get(_OLD_CODELISTS_URL).json()
            list_of_codelists = [x['name'] for x in j['codelist']]
        else:
            codelists_url = _NEW_CODELISTS_TMPL.format(
                version=version.replace('.', ''))
            list_of_codelists = requests.get(codelists_url).json()
        return list_of_codelists

    def get_codelist(codelist_name, version):
        if version in _VERY_OLD_IATI_VERSIONS:
            codelist_url = _VERY_OLD_CODELIST_TMPL.format(
                codelist_name=codelist_name)
            request = requests.get(codelist_url)
            codes = list(csv.DictReader(
                [x for x in request.iter_lines()]))
            version_codelist = {'data': codes}
        elif version in _OLD_IATI_VERSIONS:
            codelist_url = _OLD_CODELIST_TMPL.format(
                codelist_name=codelist_name)
            request = requests.get(codelist_url)
            codes = list(csv.DictReader(
                [x for x in request.iter_lines()]))
            version_codelist = {'data': codes}
        else:
            codelist_url = _NEW_CODELIST_TMPL.format(
                codelist_name=codelist_name,
                version=version.replace('.', ''))
            version_codelist = requests.get(codelist_url).json()
        return version_codelist

    path = join(CONFIG['paths']['standard'], 'codelists')
    shutil.rmtree(path, ignore_errors=True)
    makedirs(path)

    logging.getLogger(__name__).info(
        'Downloading IATI Standard codelists...')

    codelist_versions_by_name = defaultdict(list)
    all_versions = helpers.get_iati_versions()
    for version in all_versions:
        for codelist_name in get_list_of_codelists(version):
            codelist_versions_by_name[codelist_name].append(version)

    with open(join(path, 'codelists.json'), 'w') as handler:
        json.dump(codelist_versions_by_name, handler)

    for codelist_name, versions in codelist_versions_by_name.items():
        codelist = None
        for version in versions:
            version_codelist = get_codelist(codelist_name, version)

            if version_codelist.get('attributes', {}).get('embedded') == '0':
                codelist = version_codelist
                codelist['data'] = OrderedDict(
                    [(x['code'], x) for x in codelist['data']])
                break

            if codelist is None:
                codelist = {
                    'attributes': version_codelist.get('attributes', {}),
                    'metadata': version_codelist.get('metadata', {}),
                    'data': OrderedDict(),
                }

            for item in version_codelist['data']:
                if item['code'] not in codelist['data']:
                    item['from'] = version
                    item['until'] = version
                    codelist['data'][item['code']] = item
                else:
                    current_item = codelist['data'][item['code']]
                    if version < current_item['from']:
                        current_item['from'] = version
                    if version > current_item['until']:
                        current_item['until'] = version
                    codelist['data'][item['code']] = current_item

        with open(join(path, codelist_name + '.json'), 'w') as handler:
            json.dump(codelist, handler)

    _get_codelist_mappings(all_versions)


def schemas():
    path = join(CONFIG['paths']['standard'], 'schemas')
    shutil.rmtree(path, ignore_errors=True)
    makedirs(path)

    versions_url = 'https://iatistandard.org/reference_downloads/' + \
                   '201/codelists/downloads/clv2/json/en/' + \
                   'Version.json'
    versions = [d['code'] for d in requests.get(versions_url).json()['data']]
    versions.reverse()

    logging.getLogger(__name__).info('Downloading IATI Standard schemas...')
    filenames = ['iati-activities-schema.xsd', 'iati-organisations-schema.xsd',
                 'iati-common.xsd', 'xml.xsd']
    tmpl = 'https://raw.githubusercontent.com/IATI/IATI-Schemas/' + \
           'version-{version}/{filename}'
    for version in versions:
        version_path = version.replace('.', '')
        makedirs(join(path, version_path))
        for filename in filenames:
            url = tmpl.format(version=version, filename=filename)
            request = requests.get(url)
            filepath = join(path, version_path, filename)
            with open(filepath, 'wb') as handler:
                handler.write(request.content)


def standard():
    schemas()
    codelists()



from setuptools import setup, find_packages
from os.path import abspath, dirname, join


path = abspath(dirname(__file__))
with open(join(path, 'README.rst')) as f:
    readme = f.read()

data = {}
with open(join(path, 'iatikit', '__version__.py')) as f:
    exec(f.read(), data)

setup(
    name='iatikit',
    description='A toolkit for using IATI data.',
    url='https://iatikit.readthedocs.io',
    author='Andy Lulham',
    author_email='a.lulham@gmail.com',
    version=data.get('__version__'),
    packages=find_packages(),
    license='MIT',
    keywords='IATI',
    long_description=readme,
    install_requires=[
        'configparser',
        'lxml',
        'requests',
        'unicodecsv',
        'future',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    setup_requires=['pytest-runner'],
)



from io import BytesIO
import os
from os.path import abspath, dirname, join
import shutil
import tempfile
from unittest import TestCase
import zipfile

from mock import patch

from iatikit.utils import download
from iatikit.utils.config import CONFIG


class MockRequest():
    def __init__(self, url, stream=False):
        registry_path = join(dirname(abspath(__file__)),
                             'fixtures', 'registry')
        self.raw = BytesIO()
        with zipfile.ZipFile(self.raw, 'w') as ziph:
            for root, _, files in os.walk(registry_path):
                for file in files:
                    fullpath = join(root, file)
                    ziph.write(fullpath, fullpath[len(registry_path):])
        self.raw.seek(0)


class TestDownloadData(TestCase):
    def setUp(self):
        self.data_path = tempfile.mkdtemp(dir=dirname(abspath(__file__)))
        config_dict = {'paths': {'registry': self.data_path}}
        CONFIG.read_dict(config_dict)

    @patch('requests.get', MockRequest)
    def test_download_data(self):
        def list_files(path):
            all_files = []
            for root, _, files in os.walk(path):
                for file in files:
                    all_files.append(join(root, file)[len(path):])
            return all_files

        download.data()

        registry_path = join(dirname(abspath(__file__)),
                             'fixtures', 'registry')
        source_files = list_files(registry_path)
        dest_files = list_files(self.data_path)

        assert len(source_files) == len(dest_files)
        for dest_file in dest_files:
            assert dest_file in source_files

    def tearDown(self):
        shutil.rmtree(self.data_path, ignore_errors=True)



from os.path import abspath, dirname, join
import shutil
import tempfile
from unittest import TestCase

import pytest

import iatikit
from iatikit.standard.codelist import CodelistSet, Codelist
from iatikit.utils.exceptions import NoCodelistsError
from iatikit.utils.config import CONFIG


class TestNoCodelists(TestCase):
    def setUp(self):
        self.empty_path = tempfile.mkdtemp(dir=dirname(abspath(__file__)))
        config_dict = {'paths': {'standard': self.empty_path}}
        CONFIG.read_dict(config_dict)

    def test_no_codelists(self):
        with pytest.raises(NoCodelistsError):
            CodelistSet()

    def tearDown(self):
        shutil.rmtree(self.empty_path, ignore_errors=True)


class TestCodelistSet(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestCodelistSet, self).__init__(*args, **kwargs)
        standard_path = join(dirname(abspath(__file__)),
                             'fixtures', 'standard')
        config_dict = {'paths': {'standard': standard_path}}
        CONFIG.read_dict(config_dict)
        self.codelists = CodelistSet()

    def test_codelists_iter(self):
        codelist_slugs = [
            'ActivityStatus',
            'Sector',
            'SectorCategory',
            'SectorVocabulary',
            'Vocabulary',
        ]
        assert len([x for x in self.codelists]) == 5
        for codelist in self.codelists:
            assert codelist.slug in codelist_slugs

    def test_codelists_shortcut(self):
        codelists = iatikit.codelists()
        assert(len(codelists)) == 5

    def test_codelists_filter_version(self):
        codelist_slugs = [
            'Sector',
            'SectorCategory',
            'Vocabulary',
        ]
        codelists = [x for x in self.codelists.where(version='1.01')]
        assert len(codelists) == 3
        for codelist in codelists:
            assert codelist.slug in codelist_slugs

    def test_codelists_filter_slug(self):
        sector_codelist = self.codelists.find(slug='Sector')
        assert sector_codelist.slug == 'Sector'


class TestCodelist(TestCase):
    def __init__(self, *args, **kwargs):
        super(TestCodelist, self).__init__(*args, **kwargs)
        standard_path = join(dirname(abspath(__file__)),
                             'fixtures', 'standard')
        config_dict = {'paths': {'standard': standard_path}}
        CONFIG.read_dict(config_dict)
        self.codelist = Codelist('Sector', '1.05')

    def test_codelist_name(self):
        assert self.codelist.name == 'DAC 5 Digit Sector'

    def test_codelist_description(self):
        description = 'Sector codelist description'
        assert self.codelist.description == description

    def test_codelist_url(self):
        url = 'http://www.oecd.org/dac/stats/dacandcrscodelists.htm'
        assert self.codelist.url == url

    def test_codelist_complete(self):
        assert self.codelist.complete is True

    def test_codelist_iter(self):
        items = [x for x in self.codelist]
        assert len(items) == 3

    def test_codelist_repr(self):
        assert str(self.codelist) == '<Codelist (Sector v1.05)>'

    def test_codelist_filter_category(self):
        codelist_item_names = [
            'Media and free flow of information',
            'Free flow of information',
        ]
        codelist_items = [x for x in self.codelist.filter(category='151')]
        assert len(codelist_items) == 2
        for codelist_item in codelist_items:
            assert codelist_item.name in codelist_item_names

    def test_codelist_item(self):
        codelist_item = self.codelist.get('73010')
        item_repr = '<CodelistItem (Reconstruction relief and ' + \
                    'rehabilitation (73010))>'
        assert str(codelist_item) == item_repr
